/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    removeUnusedFrames.cpp
** AUTHOR(S):   Olga Russakovsky <olga@cs.stanford.edu>
** DESCRIPTION:
**   Quick application for removing unused frames from a groundtruth file
** based on the image sequence file
*****************************************************************************/

#include <cstdlib>
#include <string>
#include <limits>
#include <iomanip>
#include <fstream>
#include <map>
#include <vector>

#include "svlBase.h"
#include "svlVision.h"

using namespace std;

void usage()
{
    cerr << SVL_USAGE_HEADER << endl;
    cerr << "USAGE: ./removeUnusuedFrames <objects xml> <imageSequence>" << endl;
}


#define NUM_REQUIRED_PARAMETERS 2

int main(int argc, char **argv)
{
  char *inputFilename;
  char *imageSeq;

  char **args = argv + 1;
  while (--argc > NUM_REQUIRED_PARAMETERS) {
    if (**args == '-') {
      SVL_LOG(SVL_LOG_FATAL, "unrecognized option " << *args);
    } else {
      break;
    }
    args++;
  }

  if (argc < NUM_REQUIRED_PARAMETERS) {
    usage();
    return -1;
  }

  inputFilename = args[0];
  imageSeq = args[1];

  svlObject2dSequence objects;
  SVL_LOG(SVL_LOG_VERBOSE, "Reading from " << inputFilename);
	
  if (!readObject2dSequence(inputFilename, objects)) {
    SVL_LOG(SVL_LOG_ERROR, "...failed to read " << inputFilename);
  }

  svlImageSequence sequence;
  SVL_ASSERT_MSG(strExtension(imageSeq).compare("xml") == 0, imageSeq << " is not an image sequence");
  sequence.load(imageSeq);
 
  // decide which of the current frames to keep
    map<string, bool> keep;
    for (svlObject2dSequence::const_iterator it = objects.begin(); it != objects.end(); ++it) {
      keep[it->first] = false;
    }

  for (size_t level = 0; level < sequence.size(); level++) {
    string name = strBaseName(sequence[level]);
    keep[name] = true;
  }

  // delete all frames from video that weren't present in the given file
  for (svlObject2dSequence::iterator it = objects.begin(); it != objects.end(); ++it) {
    if (keep[it->first]) continue;
    objects.erase(it);
  }

  writeObject2dSequence(inputFilename, objects);

}

