//todo-- add full lasik header
//author-- Ian Goodfellow <ia3n@cs.stanford.edu>
//description-- a pure virtual class that other point cloud classes should subclass

#pragma once

#include "svlVision.h"

class svlPointColourer;

enum svlRenderType
  {
    SVL_RENDER_POINTS, //just renders points in isolation
    SVL_RENDER_STRUCTURED_SURFACE, //uses the structure of the data to fit a surface, renders that
    SVL_RENDER_STRUCTURED_SURFACE_PRETTY, //like above, but attempts to automatically discard some polygons where it looks like there should be gaps in the mesh
  };



class svlAbstractCloud
{
 public:

  svlAbstractCloud();
  virtual ~svlAbstractCloud();

  virtual unsigned size() const = 0;
  virtual svlPoint3d getPoint(unsigned idx) const = 0;

  virtual void render(svlRenderType renderType, double option = 0.1) const = 0; //option can be, i.e., the max depth change per polygon for RENDER_STRUCTURED_SURFACE_PRETTY


 public:
  //Callbacks to be called per point regardless of renderType
  vector<const svlPointColourer *> universalCallbacks;
  //Callbacks to be called per point only for point rendering
  vector<const svlPointColourer *> pointCallbacks;
  //Callbacks to be called per point only for surface rendering
  vector<const svlPointColourer *> surfaceCallbacks;

  //Determine whether "bad" vertices will be recalculated on next
  //rendering pass of type SVL_RENDER_STRUCTURED_SURFACE_PRETTY
  mutable bool cloudUpdated; //defaults to true
  bool assumeCloudAlwaysUpdated; //defaults to false

 protected:

  void preprocess(svlRenderType renderType) const;
  void process(const svlPoint3d &, bool pointCallbacks, bool surfaceCallbacks) const;

};
